import os
import sys

import pyglet
import kytten

import withgui
from withtk import HasProperties

class GUI(object):
    def __init__(self, spec):
        # base pyglet setup
        self.window = pyglet.window.Window(caption='withgui')
        self.gui = self

        theme = kytten.Theme(os.path.join(os.getcwd(), 'theme'), override={
            "gui_color": [128, 128, 128, 255],
            "font_size": 14
        })
	top_frame = kytten.VerticalLayout([Implementation.create(child, self).kytten_widget
            for child in spec.children])

        self.dialog = kytten.Dialog(top_frame, window=self.window, theme=theme)
        self.dialog.do_layout()
        self.dialog.needs_layout = True
        self.window.set_size(self.dialog.width, self.dialog.height)

        self.window.push_handlers(self)

    def on_draw(self):
        self.window.clear()
        self.dialog.draw()

    def update(self, dt):
	self.dialog.on_update(dt)

    def run(self):
        pyglet.clock.schedule(self.update)
        pyglet.app.run()

    def stop(self, data=None):
        pyglet.app.stop()
        return data

class Implementation(HasProperties):
    '''Wrapper around a kytten widget allowing read/write access
    to the widget's properties including its current value.
    '''
    implementation_classes = {}
    @classmethod
    def register_class(cls, klass, name=None):
        if name is None:
            name = klass.__name__
        cls.implementation_classes[name] = klass

    @classmethod
    def create(cls, spec, parent):
        klass = cls.implementation_classes[spec.__class__.__name__]
        return klass(spec, parent)

    def __init__(self, spec, parent):
        self.spec = spec
        self.parent = parent
        self.gui = parent.gui
        self.kytten_widget = None
        self.args = self.generate_properties(spec)

        self.create_widget()

        # set up event handlers
        if 'on_mouse' in spec.event_handlers:
            TODO
            spec.event_handlers['on_mouse']

        self.spec.implementation = self

    def create_widget(self):
        '''Create the underlying kytten_widget 
        '''
        raise NotImplementeError('create_widget must be implemented')

    def set_value(self, value):
        # TODO no value modification in kytten yet
        TODO
    def get_value(self):
        return self.kytten_widget.get_value()
    def set_property(self, name, value):
        name, value = self.translate_value(name, value)
        if name == 'value':
            self.set_value(value)
            return
        if name == 'x':
            self.kytten_widget.x = value
            self.kytten_widget.place(x=value)
        elif name == 'y':
            self.kytten_widget.y = value
            self.kytten_widget.place(y=value)
        else:
            # TODO no property modification in kytten yet
            TODO
    def get_property(self, name):
        if name == 'value':
            return self.get_value()
        elif name == 'x':
            return self.kytten_widget.x
        elif name == 'y':
            return self.kytten_widget.y
        else:
            return getattr(self.kytten_widget, name)

    def destroy(self):
        self.spec.implementation = None
        self.parent = None
        self.gui = None
        self.args = None
        self.kytten_widget.delete()
        self.kytten_widget = None

class Frame(Implementation):
    def create_widget(self):

        width = self.args.get('width')
        height = self.args.get('height')

        image = self.args.get('image')
        if image is not None:
            if 'width' is None:
                width = image.width()
            if 'height' is None:
                height = image.height()

	self.kytten_widget = kytten.FreeLayout(width, height)

        # XXX do somehing with image

        for child in spec.children:
            self.add_child(child)

    def add_child(self, spec):
            i = Implementation.create(child, self)
            anchor = i.args['anchor']
            x = i.args['x']
            y = i.args['y']
            self.kytten_widget.add(anchor, x, y, i.kytten_widget)
Implementation.register_class(Frame)

class Column(Implementation):
    def create_widget(self):
	self.kytten_widget = kytten.VerticalLayout([
            Implementation.create(child, self).kytten_widget
                for child in self.spec.children])
    def add_child(self, spec):
            i = Implementation.create(child, self)
            self.add(i.kytten_widget)
Implementation.register_class(Column)

class Row(Implementation):
    def create_widget(self):
	self.kytten_widget = kytten.HorizontalLayout([
            Implementation.create(child, self).kytten_widget
                for child in self.spec.children])
    def add_child(self, spec):
            i = Implementation.create(child, self)
            self.add(i.kytten_widget)
Implementation.register_class(Row)

class Label(Implementation):
    def create_widget(self):
        self.kytten_widget = kytten.Label(self.args['value'])
Implementation.register_class(Label)

class Button(Implementation):
    def create_widget(self):
        def click(spec=self.spec):
            return spec.event_handlers['on_click'](spec)
        self.kytten_widget = kytten.Button(self.args['value'], on_click=click)
Implementation.register_class(Button)

class Selection(Implementation):
    properties = dict(
        options=('options', HasProperties.prop_passthrough),
        value=('value', HasProperties.prop_passthrough),
    )
    def create_widget(self):
        options = self.args['options']
        value = self.args.get('value', options[0])
        self.kytten_widget = kytten.Dropdown(options=options, selected=value)
Implementation.register_class(Selection)

class Text(Implementation):
    def create_widget(self):
        value = self.args.get('value', '')
        self.kytten_widget = kytten.Input(text=text)
    def set_value(self, value):
        self.kytten_widget.set_text(value)
Implementation.register_class(Text)

