import sys

import PyQt4.Qt as qt

import withgui

class GUI(object):
    def __init__(self, app):
        self.app = app

    def run(self):
        self.app.exec_()

    def stop(self, data=None):
        self.app.quit()
        return data

class WidgetWrapper(object):
    '''Wrapper around a Tkinter widget and optionally a separate Tkinter
    variable allowing read/write access to the widget's properties
    including its current value.
    '''
    def __init__(self, widget, variable):
        self.widget = widget
        self.variable = variable
    def set_property(self, name, value):
        if name == 'value':
            self.variable.set(value)
        else:
            # XXX check this is correct?
            setattr(self.widget, name, value)
    def get_property(self, name):
        if name == 'value':
            return self.variable.get()
        else:
            return getattr(self.widget, name)

# mapping withgui properties to Tkinter properties
prop_passthrough = lambda v: v
prop_color = lambda v: '#'+v.to_hex()
properties = dict(
    justify=('justify', prop_passthrough),
    foreground=('foreground', prop_color),
    background=('background', prop_color),
    value=('value', prop_passthrough),
)
label_properties = dict(
    value=('text', prop_passthrough),
)
text_properties = dict(
    value=('text', prop_passthrough),
)
def generate_properties(node, override):
    d = {}
    for k,v in node.defaults.items():
        if k in override:
            k, p = override[k]
        else:
            k, p = properties[k]
        d[k] = p(v)
    return d

class Constructor(object):
    def construct(self, node, parent=None):
        adder = getattr(self, 'add_%s'%node.__class__.__name__)
        w = adder(node, parent)
        return w

    def add_Window(self, node, parent):
        self.app = qt.QApplication(sys.argv[:1])
        self.app.setQuitOnLastWindowClosed(True)
        c = self.construct(node.children[0])
        c.setMinimumSize(200, 100)
        c.show()
        return GUI(self.app)

    def add_Vertical(self, node, parent):
        layout = QVBoxLayout(parent)
        for child in node.children:
            self.construct(child, layout)
        return layout

    def add_Label(self, node, parent, **kw):
        label = qt.QLabel()
        kw = generate_properties(node, label_properties)
        label.setText(kw.pop('text'))
        node.implementation = WidgetWrapper(label, label)
        return label

    def add_Form(self, node, parent):
        frame = tk.Frame(parent)
        frame._form = {}
        for n, row in enumerate(node.rows):
            label = ''
            if row.label is not None:
                l = self.add_Label(row.label, frame, justify=tk.RIGHT)
                l.grid(row=n, column=0, sticky=tk.E)
                label = row.label.value

            if row.widget is not None:
                # figure the name for this widget in the form
                w = self.construct(row.widget, frame)
                w.grid(row=n, column=1, sticky=tk.W)

            if row.help is not None:
                l = self.add_Label(row.help, frame, justify=tk.LEFT)
                l.grid(row=n, column=2, sticky=tk.W)

        bframe = tk.Frame(frame)
        for button in node.buttons:
            w = self.construct(button, bframe)
            w.pack(side=tk.LEFT)

        bframe.grid(row=n+1, column=1, columnspan=2, sticky=tk.W)
        return frame

    def add_Text(self, node, parent):
        kw = generate_properties(node, label_properties)
        w = tk.Entry(parent, **kw)
        node.implementation = WidgetWrapper(w, w)
        return w

    def add_Selection(self, node, parent):
        kw = generate_properties(node, {})
        var = tk.StringVar(parent)
        value = kw.pop('value')
        var.set(value[0])
        args = (parent, var) + tuple(value)
        option = tk.OptionMenu(*args, **kw)
        node.implementation = WidgetWrapper(option, var)
        return option

    def add_Help(self, node, parent):
        return self.add_Label(node, parent)

    def add_Button(self, node, parent):
        kw = generate_properties(node, label_properties)
        label = kw.pop('text')
        w = qt.QPushButton(label, parent)
        self.app.connect(w, qt.SIGNAL("clicked()"), node.event_handlers.get('on_click'))
        node.implementation = WidgetWrapper(w, w)
        return w
    add_Submit = add_Button
    add_Cancel = add_Button


